from setuptools import setup
from os import path, chdir, getcwd
from glob import glob

chdir(path.abspath(path.dirname(__file__)))

def read(filename):
    try:
        with open(filename, 'rb') as fp:
            data = fp.read().decode('utf-8')
    except UnicodeDecodeError:
            with open(filename, 'r') as fp:
                data = fp.read()
    return data

def find_version(file_paths):
    version_file = read(file_paths)
    import re
    version_match = re.search(
        r"^__version__ = ['\"]([^'\"]*)['\"]",
        version_file,
        re.M,
    )
    if version_match:
        return version_match.group(1)

    raise RuntimeError("Unable to find version string.")

def get_package_data_list():
    from sys import version_info
    from platform import machine, system

    if system().lower() == "linux":

        if machine().lower() == "x86_64":
            ends_with = "linux-gnu.so"

        elif machine().lower().startswith("arm"):
            ends_with = "arm-linux-gnueabihf.so"

    if system().lower() == "windows":

        if machine().lower() == "amd64":
            ends_with = "win_amd64.pyd"

    if system().lower() == "darwin":
        ends_with = "darwin.so"


    _find_all_files = []
    from os import walk
    for root, subfiles, files in walk('src'):
        for file in files:
            if file.endswith(ends_with):
                absfile = path.abspath( path.join( root, file ) )
                _find_all_files.append( absfile )

    return _find_all_files

setup(
    name='sourcedefender',
    version=find_version('src/__init__.py'),
    python_requires="!=2.*,>=3.6",
    description='Advanced encryption protecting your python codebase.',
    long_description=read(path.join(getcwd(),'README.md')) + '\n',
    long_description_content_type="text/markdown",
    author='SOURCEdefender',
    author_email="hello@sourcedefender.co.uk",
    keywords="encryption source aes",
    packages=[ 'sourcedefender' ],
    package_dir={ 'sourcedefender': 'src' },
    package_data={ 'sourcedefender': get_package_data_list() },
    install_requires=open("requirements.txt").read().splitlines(),
    url="https://sourcedefender.co.uk/",
    license='Other/Proprietary License',
    entry_points={
        'console_scripts': [
        'sourcedefender = sourcedefender.encrypt:main',
        ]
    },
    zip_safe=True,
    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 5 - Production/Stable',

        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Other Audience',
        'Intended Audience :: Science/Research',
        'Intended Audience :: System Administrators',

        'Topic :: Security',
        'Topic :: Security :: Cryptography',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Software Distribution',
        'Topic :: Utilities',

        'License :: Other/Proprietary License',

        'Operating System :: MacOS',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',

        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: Implementation :: CPython'
    ],
)
